from .dataclasses import RemoteAppConfig
from .gr_application import RemoteGradioApplication
from fastapi import FastAPI
import json
import gradio as gr


def create_gradio2api(
    configs: list[RemoteAppConfig | dict],
    fastapi_config: dict = {},
    gradio_app_mount_path: str = "/gui",
) -> tuple[FastAPI, list[RemoteGradioApplication | None]]:
    app = FastAPI(**fastapi_config)
    remote_gr_applications: list[RemoteGradioApplication] = []

    for config in configs:
        if isinstance(config, RemoteAppConfig):
            config = config.model_dump()
        ga_success_flag = None
        try:
            ga = RemoteGradioApplication(**config)
            ga_success_flag = True
        except Exception as e:
            print("="*30)
            print("[warning] gradio application creation failed with config:")
            print(json.dumps(config, indent=2))
            print("-"*30)
            print(e)
            print("="*30)

        if ga_success_flag is True:
            app.include_router(ga.router)
            remote_gr_applications.append(ga)
        else:
            remote_gr_applications.append(None)

    tabbed_interface = gr.TabbedInterface(
        [ga.ui for ga in remote_gr_applications if ga is not None],
    )

    app = gr.mount_gradio_app(
        app,
        tabbed_interface,
        gradio_app_mount_path,
    )

    return [app, remote_gr_applications]
