from pydantic import BaseModel, Field
from pathlib import Path
from typing import Any, Literal, Optional
from gradio_client.client import DEFAULT_TEMP_DIR


class ExtraModel(BaseModel):
    model_config = {
        "extra": "allow"
    }


class GradioClientExtraConfig(ExtraModel):
    """Gradio Client Configuration"""
    hf_token: str | None | Literal[False] = False
    max_workers: int = 40
    verbose: bool = True
    auth: tuple[str, str] | None = None
    httpx_kwargs: dict[str, Any] | None = None


class RouterConfig(ExtraModel):
    """Router Configuration"""
    prefix: str = "/"
    tags: list[str] = []


class RemoteAppConfig(BaseModel):
    """Remote Configuration"""
    src: str
    client_kwargs: GradioClientExtraConfig = GradioClientExtraConfig()
    router_kwargs: RouterConfig = RouterConfig()


class Gradio2APIConfig(BaseModel):
    configs: list[RemoteAppConfig]
